@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.122 2011/06/15 17:54:32 stroeder Exp $
+\$Id: ldapobject.py,v 1.124 2011/07/21 17:40:11 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -32,7 +32,7 @@ if __debug__:
   # Tracing is only supported in debugging mode
   import traceback
 
-import sys,time,_ldap,ldap,ldap.functions
+import sys,time,pprint,_ldap,ldap,ldap.functions
 
 from ldap.schema import SCHEMA_ATTRS
 from ldap.controls import LDAPControl,DecodeControlTuples,RequestControlTuples
@@ -85,21 +85,20 @@ class SimpleLDAPObject:
     self._ldap_object_lock.acquire()
     if __debug__:
       if self._trace_level>=1:
-        self._trace_file.write('*** %s - %s (%s,%s)\n' % (
+        self._trace_file.write('*** %s - %s\n%s\n' % (
           self._uri,
-          self.__class__.__name__+'.'+func.__name__,
-          repr(args),repr(kwargs)
+          '.'.join((self.__class__.__name__,func.__name__)),
+          pprint.pformat((args,kwargs))
         ))
         if self._trace_level>=9:
           traceback.print_stack(limit=self._trace_stack_limit,file=self._trace_file)
+    diagnostic_message_success = None
     try:
       try:
         result = func(*args,**kwargs)
         if __debug__ and self._trace_level>=2:
           if func.__name__!="unbind_ext":
             diagnostic_message_success = self._l.get_option(ldap.OPT_DIAGNOSTIC_MESSAGE)
-          else:
-            diagnostic_message_success = None
       finally:
         self._ldap_object_lock.release()
     except LDAPError,e:
@@ -110,7 +109,7 @@ class SimpleLDAPObject:
       if __debug__ and self._trace_level>=2:
         if not diagnostic_message_success is None:
           self._trace_file.write('=> diagnosticMessage: %s\n' % (repr(diagnostic_message_success)))
-        self._trace_file.write('=> result: %s\n' % (repr(result)))
+        self._trace_file.write('=> result:\n%s\n' % (pprint.pformat(result)))
     return result
 
   def __setattr__(self,name,value):
