@@ -3,7 +3,7 @@ ldap.schema.subentry -  subschema subentry handling
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: subentry.py,v 1.25 2010/04/30 08:39:38 stroeder Exp $
+\$Id: subentry.py,v 1.30 2011/07/08 10:14:53 stroeder Exp $
 """
 
 import ldap.cidict,ldap.schema
@@ -24,35 +24,93 @@ for _name in dir():
 SCHEMA_ATTRS = SCHEMA_CLASS_MAPPING.keys()
 
 
+class SubschemaError(ValueError):
+  pass
+
+
+class OIDNotUnique(SubschemaError):
+
+  def __init__(self,desc):
+    self.desc = desc
+
+  def __str__(self):
+    return 'OID not unique for %s' % (self.desc)
+
+
+class NameNotUnique(SubschemaError):
+
+  def __init__(self,desc):
+    self.desc = desc
+
+  def __str__(self):
+    return 'NAME not unique for %s' % (self.desc)
+
+
 class SubSchema:
-    
-  def __init__(self,sub_schema_sub_entry):
-      """
-      sub_schema_sub_entry
-          Dictionary containing the sub schema sub entry
-      """
-      # Initialize all dictionaries
-      self.name2oid = {}
-      self.sed = {}
-      for c in SCHEMA_CLASS_MAPPING.values():
-        self.name2oid[c] = ldap.cidict.cidict()
-        self.sed[c] = {}
 
-      e = ldap.cidict.cidict(sub_schema_sub_entry)
+  def __init__(self,sub_schema_sub_entry,check_uniqueness=1):
+    """
+    sub_schema_sub_entry
+        Dictionary containing the sub schema sub entry
+    check_uniqueness
+        Defines whether uniqueness of OIDs and NAME is checked.
+        0 no check
+        1 check but add schema description with work-around
+        2 check and raise exception if non-unique OID or NAME is found
+    """
+
+    # Initialize all dictionaries
+    self.name2oid = {}
+    self.sed = {}
+    self.non_unique_oids = {}
+    self.non_unique_names = {}
+    for c in SCHEMA_CLASS_MAPPING.values():
+      self.name2oid[c] = ldap.cidict.cidict()
+      self.sed[c] = {}
+      self.non_unique_names[c] = set()
+
+    # Transform entry dict to case-insensitive dict
+    e = ldap.cidict.cidict(sub_schema_sub_entry)
+
+    # Build the schema registry in dictionaries
+    for attr_type in SCHEMA_ATTRS:
+
+      for attr_value in filter(None,e.get(attr_type,[])):
+
+        se_class = SCHEMA_CLASS_MAPPING[attr_type]
+        se_instance = se_class(attr_value)
+        se_id = se_instance.get_id()
+
+        if check_uniqueness and se_id in self.sed[se_class]:
+            self.non_unique_oids[se_id] = None
+            if check_uniqueness==1:
+              # Add to subschema by adding suffix to ID
+              suffix_counter = 1
+              new_se_id = se_id
+              while new_se_id in self.sed[se_class]:
+                new_se_id = ';'.join((se_id,str(suffix_counter)))
+                suffix_counter += 1
+              else:
+                se_id = new_se_id
+            elif check_uniqueness>=2:
+              raise OIDNotUnique(attr_value)
+
+        # Store the schema element instance in the central registry
+        self.sed[se_class][se_id] = se_instance
+
+        if hasattr(se_instance,'names'):
+          for name in se_instance.names:
+            if check_uniqueness and name in self.name2oid[se_class]:
+              self.non_unique_names[se_class][se_id] = None
+              raise NameNotUnique(attr_value)
+            else:
+              self.name2oid[se_class][name] = se_id
+
+    # Turn dict into list maybe more handy for applications
+    self.non_unique_oids = self.non_unique_oids.keys()
+
+    return # subSchema.__init__()
 
-      # Build the schema registry
-      for attr_type in SCHEMA_ATTRS:
-        if not e.has_key(attr_type) or \
-           not e[attr_type]:
-          continue
-        for attr_value in filter(None,e[attr_type]):
-          se_class = SCHEMA_CLASS_MAPPING[attr_type]
-          se_instance = se_class(attr_value)
-          self.sed[se_class][se_instance.get_id()] = se_instance
-          if hasattr(se_instance,'names'):
-            for name in se_instance.names:
-              self.name2oid[se_class][name] = se_instance.get_id()
-      return # subSchema.__init__()
 
   def ldap_entry(self):
     """
@@ -90,7 +148,7 @@ class SubSchema:
     else:
       result = avail_se.keys()
     return result
-    
+
 
   def tree(self,schema_element_class,schema_element_filters=None):
     """
@@ -116,7 +174,7 @@ class SubSchema:
           se_oid,schema_element_class.__name__,se_obj.__class__
         )
       for s in se_obj.sup or ('_',):
-        sup_oid = self.name2oid[schema_element_class].get(s,s)
+        sup_oid = self.getoid(schema_element_class,s)
         try:
           tree[sup_oid].append(se_oid)
         except:
@@ -124,12 +182,23 @@ class SubSchema:
     return tree
 
 
-  def getoid(self,se_class,nameoroid):
+  def getoid(self,se_class,nameoroid,raise_keyerror=0):
     """
     Get an OID by name or OID
     """
-    se_oid = nameoroid.split(';')[0].strip()
-    return self.name2oid[se_class].get(se_oid,se_oid)
+    nameoroid_stripped = nameoroid.split(';')[0].strip()
+    if nameoroid_stripped in self.sed[se_class]:
+      # name_or_oid is already a registered OID
+      return nameoroid_stripped
+    else:
+      try:
+        result_oid = self.name2oid[se_class][nameoroid_stripped]
+      except KeyError:
+        if raise_keyerror:
+          raise KeyError('No registered %s-OID for nameoroid %s' % (se_class.__name__,repr(nameoroid_stripped)))
+        else:
+          result_oid = nameoroid_stripped
+    return result_oid
 
 
   def get_inheritedattr(self,se_class,nameoroid,name):
@@ -137,7 +206,7 @@ class SubSchema:
     Get a possibly inherited attribute specified by name
     of a schema element specified by nameoroid.
     Returns None if class attribute is not set at all.
-    
+
     Raises KeyError if no schema element is found by nameoroid.
     """
     se = self.sed[se_class][self.getoid(se_class,nameoroid)]
@@ -150,11 +219,21 @@ class SubSchema:
     return result
 
 
-  def get_obj(self,se_class,nameoroid,default=None):
+  def get_obj(self,se_class,nameoroid,default=None,raise_keyerror=0):
     """
     Get a schema element by name or OID
     """
-    return self.sed[se_class].get(self.getoid(se_class,nameoroid),default)
+    se_oid = self.getoid(se_class,nameoroid)
+    try:
+      se_obj = self.sed[se_class][se_oid]
+    except KeyError:
+      if raise_keyerror:
+        raise KeyError('No ldap.schema.%s instance with nameoroid %s and se_oid %s' % (
+          se_class.__name__,repr(nameoroid),repr(se_oid))
+        )
+      else:
+        se_obj = default
+    return se_obj
 
 
   def get_inheritedobj(self,se_class,nameoroid,inherited=None):
@@ -242,17 +321,17 @@ class SubSchema:
         which has to be matched
     raise_keyerror
         All KeyError exceptions for non-existent schema elements
-	are ignored
+        are ignored
     ignore_dit_content_rule
-	A DIT content rule governing the structural object class
-	is ignored
+        A DIT content rule governing the structural object class
+        is ignored
     """
     AttributeType = ldap.schema.AttributeType
     ObjectClass = ldap.schema.ObjectClass
 
     # Map object_class_list to object_class_oids (list of OIDs)
     object_class_oids = [
-      self.name2oid[ObjectClass].get(o,o)
+      self.getoid(ObjectClass,o)
       for o in object_class_list
     ]
     # Initialize
@@ -284,28 +363,14 @@ class SubSchema:
       assert hasattr(object_class,'must'),ValueError(object_class_oid)
       assert hasattr(object_class,'may'),ValueError(object_class_oid)
       for a in object_class.must:
-        try:
-          at_obj = self.sed[AttributeType][self.name2oid[AttributeType].get(a,a)]
-        except KeyError:
-          if raise_keyerror:
-            raise
-          else:
-            r_must[a] = None
-        else:
-          r_must[at_obj.oid] = at_obj
+        se_oid = self.getoid(AttributeType,a,raise_keyerror=raise_keyerror)
+        r_must[se_oid] = self.get_obj(AttributeType,se_oid,raise_keyerror=raise_keyerror)
       for a in object_class.may:
-        try:
-          at_obj = self.sed[AttributeType][self.name2oid[AttributeType].get(a,a)]
-        except KeyError:
-          if raise_keyerror:
-            raise
-          else:
-            r_may[a] = None
-        else:
-          r_may[at_obj.oid] = at_obj
+        se_oid = self.getoid(AttributeType,a,raise_keyerror=raise_keyerror)
+        r_may[se_oid] = self.get_obj(AttributeType,se_oid,raise_keyerror=raise_keyerror)
 
       object_class_oids.extend([
-        self.name2oid[ObjectClass].get(o,o)
+        self.getoid(ObjectClass,o)
         for o in object_class.sup
       ])
 
@@ -320,45 +385,24 @@ class SubSchema:
       structural_oc = self.get_structural_oc(object_class_list)
       if structural_oc:
         # Process applicable DIT content rule
-        dit_content_rule = self.get_obj(DITContentRule,structural_oc)
-        if dit_content_rule:
+        try:
+          dit_content_rule = self.get_obj(DITContentRule,structural_oc,raise_keyerror=1)
+        except KeyError:
+          # Not DIT content rule found for structural objectclass
+          pass
+        else:
           for a in dit_content_rule.must:
-            try:
-              at_obj = self.sed[AttributeType][self.name2oid[AttributeType].get(a,a)]
-            except KeyError:
-              if raise_keyerror:
-                raise
-              else:
-                r_must[a] = None
-            else:
-              r_must[at_obj.oid] = at_obj
+            se_oid = self.getoid(AttributeType,a,raise_keyerror=raise_keyerror)
+            r_must[se_oid] = self.get_obj(AttributeType,se_oid,raise_keyerror=raise_keyerror)
           for a in dit_content_rule.may:
-            try:
-              at_obj = self.sed[AttributeType][self.name2oid[AttributeType].get(a,a)]
-            except KeyError:
-              if raise_keyerror:
-                raise
-              else:
-                r_may[a] = None
-            else:
-              r_may[at_obj.oid] = at_obj
+            se_oid = self.getoid(AttributeType,a,raise_keyerror=raise_keyerror)
+            r_may[se_oid] = self.get_obj(AttributeType,se_oid,raise_keyerror=raise_keyerror)
           for a in dit_content_rule.nots:
-            a_oid = self.name2oid[AttributeType].get(a,a)
-            if not r_must.has_key(a_oid):
-              try:
-                at_obj = self.sed[AttributeType][a_oid]
-              except KeyError:
-                if raise_keyerror:
-                  raise
-              else:
-                try:
-                  del r_must[at_obj.oid]
-                except KeyError:
-                  pass
-                try:
-                  del r_may[at_obj.oid]
-                except KeyError:
-                  pass
+            a_oid = self.getoid(AttributeType,a,raise_keyerror=raise_keyerror)
+            try:
+              del r_may[a_oid]
+            except KeyError:
+              pass
 
     # Apply attr_type_filter to results
     if attr_type_filter:
@@ -385,7 +429,7 @@ class SubSchema:
 def urlfetch(uri,trace_level=0):
   """
   Fetches a parsed schema entry by uri.
-  
+
   If uri is a LDAP URL the LDAP server is queried directly.
   Otherwise uri is assumed to point to a LDIF file which
   is loaded with urllib.
