@@ -1,6 +1,6 @@
 /* constants defined for LDAP
  * See http://www.python-ldap.org/ for details.
- * $Id: constants.c,v 1.52 2011/04/11 11:23:46 stroeder Exp $ */
+ * $Id: constants.c,v 1.54 2011/06/10 21:55:45 stroeder Exp $ */
 
 #include "common.h"
 #include "constants.h"
@@ -196,6 +196,9 @@ LDAPinit_constants( PyObject* d )
 #ifdef LDAP_OPT_X_TLS_PROTOCOL_MIN
   add_int(d,OPT_X_TLS_PROTOCOL_MIN);
 #endif
+#ifdef LDAP_OPT_X_TLS_PACKAGE
+  add_int(d,OPT_X_TLS_PACKAGE);
+#endif
 #endif
   add_int(d,OPT_X_SASL_MECH);
   add_int(d,OPT_X_SASL_REALM);
@@ -262,7 +265,7 @@ LDAPinit_constants( PyObject* d )
 
   /* author */
 
-  author = PyString_FromString("python-ldap-dev@lists.sf.net");
+  author = PyString_FromString("python-ldap Project");
   PyDict_SetItemString(d, "__author__", author);
   Py_DECREF(author);
 
